@@ -35,9 +35,6 @@ const Application = new Lang.Class({
     vfunc_startup: function() {
         this.parent();
 
-        let w = new Polari.FixedSizeFrame(); // register gtype
-        w.destroy();
-
         this._chatroomManager = ChatroomManager.getDefault();
         this._accountsMonitor = AccountsMonitor.getDefault();
         this._networkMonitor = Gio.NetworkMonitor.get_default();
@@ -69,7 +66,7 @@ const Application = new Lang.Class({
             parameter_type: GLib.VariantType.new('(ssu)') },
           { name: 'message-user',
             activate: Lang.bind(this, this._onMessageUser),
-            parameter_type: GLib.VariantType.new('(ssu)') },
+            parameter_type: GLib.VariantType.new('(sssu)') },
           { name: 'paste-text',
             activate: Lang.bind(this, this._onPasteText),
             parameter_type: GLib.VariantType.new('s') },
@@ -87,11 +84,12 @@ const Application = new Lang.Class({
             create_hook: Lang.bind(this, this._userListCreateHook),
             state: GLib.Variant.new('b', false),
             accels: ['F9', '<Primary>u'] },
+          { name: 'remove-connection',
+            activate: Lang.bind(this, this._onRemoveConnection),
+            parameter_type: GLib.VariantType.new('o') },
           { name: 'edit-connection',
             activate: Lang.bind(this, this._onEditConnection),
             parameter_type: GLib.VariantType.new('o') },
-          { name: 'connections',
-            activate: Lang.bind(this, this._onListConnections) },
           { name: 'preferences',
             activate: Lang.bind(this, this._onShowPreferences) },
           { name: 'about',
@@ -248,7 +246,7 @@ const Application = new Lang.Class({
         account.update_parameters_vardict_async(asv, [], callback);
     },
 
-    _requestChannel: function(accountPath, targetType, targetId, time) {
+    _requestChannel: function(accountPath, targetType, targetId, time, callback) {
         // have this in AccountMonitor?
         let factory = Tp.AccountManager.dup().get_factory();
         let account = factory.ensure_account(accountPath, []);
@@ -275,7 +273,8 @@ const Application = new Lang.Class({
           cancellable: new Gio.Cancellable(),
           time: time,
           retry: 0,
-          originalNick: account.nickname };
+          originalNick: account.nickname,
+          callback: callback };
 
         this._pendingRequests[roomId] = requestData;
 
@@ -289,7 +288,7 @@ const Application = new Lang.Class({
         req.set_target_id(requestData.targetHandleType, requestData.targetId);
         req.set_delegate_to_preferred_handler(true);
         let preferredHandler = Tp.CLIENT_BUS_NAME_BASE + 'Polari';
-        req.ensure_channel_async(preferredHandler, requestData.cancellable,
+        req.ensure_and_observe_channel_async(preferredHandler, requestData.cancellable,
                                  Lang.bind(this,
                                            this._onEnsureChannel, requestData));
     },
@@ -309,9 +308,10 @@ const Application = new Lang.Class({
 
     _onEnsureChannel: function(req, res, requestData) {
         let account = req.account;
+        let channel = null;
 
         try {
-            req.ensure_channel_finish(res);
+            channel = req.ensure_and_observe_channel_finish(res);
         } catch (e if e.matches(Tp.Error, Tp.Error.DISCONNECTED)) {
             let error = account.connection_error;
             // If we receive a disconnect error and the network is unavailable,
@@ -333,6 +333,9 @@ const Application = new Lang.Class({
             logError(e, 'Failed to ensure channel');
         }
 
+        if (requestData.callback)
+            requestData.callback(channel);
+
         if (requestData.retry > 0)
             this._updateAccountName(account, requestData.originalNick, null);
         delete this._pendingRequests[requestData.roomId];
@@ -349,9 +352,26 @@ const Application = new Lang.Class({
     },
 
     _onMessageUser: function(action, parameter) {
-        let [accountPath, contactName, time] = parameter.deep_unpack();
+        let [accountPath, contactName, message, time] = parameter.deep_unpack();
         this._requestChannel(accountPath, Tp.HandleType.CONTACT,
-                             contactName, time);
+                             contactName, time, Lang.bind(this, this._sendMessage, message));
+    },
+
+    _sendMessage: function(channel, message) {
+        if (!message || !channel)
+            return;
+
+        let TpMessage = Tp.ClientMessage.new_text(Tp.ChannelTextMessageType.NORMAL,
+                                                  message);
+        channel.send_message_async(TpMessage, 0, Lang.bind(this,
+            function(c, res) {
+            try {
+                c.send_message_finish(res);
+            } catch(e) {
+                // TODO: propagate to user
+                logError(e, 'Failed to send message')
+            }
+        }));
     },
 
     _onPasteText: function(action, parameter) {
@@ -395,20 +415,26 @@ const Application = new Lang.Class({
         action.change_state(GLib.Variant.new('b', !state.get_boolean()));
     },
 
-    _onListConnections: function() {
-        if (this._connectionsDialog) {
-            this._connectionsDialog.widget.present();
-            return;
-        }
+    _onRemoveConnection: function(action, parameter){
+        let accountPath = parameter.deep_unpack();
+        let factory = Tp.AccountManager.dup().get_factory();
+        let account = factory.ensure_account(accountPath, []);
+        account.set_enabled_async(false, Lang.bind(this,
+            function() {
+                let label = _("%s removed.").format(account.display_name);
+                let n = new AppNotifications.UndoNotification(label);
+                this.notificationQueue.addNotification(n);
 
-        this.hold();
-        this._connectionsDialog = new Connections.ConnectionsDialog();
-        this._connectionsDialog.widget.show();
-        this._connectionsDialog.widget.connect('response',
-            Lang.bind(this, function(widget) {
-                widget.destroy();
-                this._connectionsDialog = null;
-                this.release();
+                n.connect('closed', function() {
+                    account.remove_async(function(a, res) {
+                        a.remove_finish(res); // TODO: Check for errors
+                    });
+                });
+                n.connect('undo', function() {
+                    account.set_enabled_async(true, function(a, res) {
+                        a.set_enabled_finish(res); // TODO: Check for errors
+                    });
+                });
             }));
     },
 
