@@ -11,6 +11,7 @@ const Tpl = imports.gi.TelepathyLogger;
 
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
+const Signals = imports.signals;
 const Utils = imports.utils;
 
 const MAX_NICK_CHARS = 8;
@@ -28,7 +29,10 @@ const STATUS_NOISE_MAXIMUM = 4;
 const NUM_INITIAL_LOG_EVENTS = 50; // number of log events to fetch on start
 const NUM_LOG_EVENTS = 10; // number of log events to fetch when requesting more
 
-const MARGIN = 6;
+const MARGIN = 14;
+const NICK_SPACING = 14; // space after nicks, matching the following elements
+                         // of the nick button in the entry area:
+                         // 8px padding + 6px spacing
 
 const NICKTAG_PREFIX = 'nick';
 
@@ -233,6 +237,12 @@ const ChatView = new Lang.Class({
         this._pendingLogs = [];
         this._statusCount = { left: 0, joined: 0, total: 0 };
 
+        this._room.account.connect('notify::nickname', Lang.bind(this,
+            function() {
+                this._updateMaxNickChars(this._room.account.nickname.length);
+            }));
+        this._updateMaxNickChars(this._room.account.nickname.length);
+
         let isRoom = room.type == Tp.HandleType.ROOM;
         let target = new Tpl.Entity({ type: isRoom ? Tpl.EntityType.ROOM
                                                    : Tpl.EntityType.CONTACT,
@@ -490,6 +500,19 @@ const ChatView = new Lang.Class({
         return Object.keys(this._pending).length;
     },
 
+    get maxNickChars() {
+        return this._maxNickChars;
+    },
+
+    _updateMaxNickChars: function(length) {
+        if (length <= this._maxNickChars)
+            return;
+
+        this._maxNickChars = length;
+        this.emit('max-nick-chars-changed');
+        this._updateIndent();
+    },
+
     _updateIndent: function() {
         let context = this._view.get_pango_context();
         let metrics = context.get_metrics(null, null);
@@ -497,11 +520,13 @@ const ChatView = new Lang.Class({
                                  metrics.get_approximate_digit_width());
         let pixelWidth = Pango.units_to_double(charWidth);
 
+        let totalWidth = this._maxNickChars * pixelWidth + NICK_SPACING;
+
         let tabs = Pango.TabArray.new(1, true);
-        tabs.set_tab(0, Pango.TabAlign.LEFT, this._maxNickChars * pixelWidth);
+        tabs.set_tab(0, Pango.TabAlign.LEFT, totalWidth);
         this._view.tabs = tabs;
-        this._view.indent = -this._maxNickChars * pixelWidth;
-        this._view.left_margin = MARGIN + this._maxNickChars * pixelWidth;
+        this._view.indent = -totalWidth;
+        this._view.left_margin = MARGIN + totalWidth;
     },
 
     _updateActive: function() {
@@ -539,7 +564,7 @@ const ChatView = new Lang.Class({
 
     _onScroll: function(w, event) {
         let [hasDir, dir] = event.get_scroll_direction();
-        if (hasDir && dir != Gtk.ScrollDirection.UP)
+        if (hasDir && dir != Gdk.ScrollDirection.UP)
             return Gdk.EVENT_PROPAGATE;
 
         let [hasDeltas, dx, dy] = event.get_scroll_deltas();
@@ -734,6 +759,10 @@ const ChatView = new Lang.Class({
 
         this._channel = this._room.channel;
 
+        let nick = this._channel ? this._channel.connection.self_contact.alias
+                                 : this._room.account.nickname;
+        this._updateMaxNickChars(nick.length);
+
         if (!this._channel)
             return;
 
@@ -1105,10 +1134,7 @@ const ChatView = new Lang.Class({
         }
         state.lastTimestamp = message.timestamp;
 
-        if (message.nick.length > this._maxNickChars) {
-            this._maxNickChars = message.nick.length;
-            this._updateIndent();
-        }
+        this._updateMaxNickChars(message.nick.length);
 
         let tags = [];
         if (isAction) {
@@ -1228,3 +1254,4 @@ const ChatView = new Lang.Class({
             buffer.apply_tag(tags[i], start, iter);
     }
 });
+Signals.addSignalMethods(ChatView.prototype);
