@@ -1,6 +1,7 @@
 const Gdk = imports.gi.Gdk;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
+const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
 const Tp = imports.gi.TelepathyGLib;
 
@@ -19,6 +20,76 @@ const Utils = imports.utils;
 const CONFIGURE_TIMEOUT = 100; /* ms */
 
 
+const FixedSizeFrame = new Lang.Class({
+    Name: 'FixedSizeFrame',
+    Extends: Gtk.Frame,
+    Properties: {
+        height: GObject.ParamSpec.int('height',
+                                      'height',
+                                      'height',
+                                      GObject.ParamFlags.READWRITE,
+                                      -1, GLib.MAXINT32, -1),
+        width: GObject.ParamSpec.int('width',
+                                     'width',
+                                     'width',
+                                     GObject.ParamFlags.READWRITE,
+                                     -1, GLib.MAXINT32, -1)
+    },
+
+    _init: function(params) {
+        this._height = -1;
+        this._width = -1;
+
+        this.parent(params);
+    },
+
+    _queueRedraw: function() {
+        let child = this.get_child();
+        if (child)
+            child.queue_resize();
+        this.queue_draw();
+    },
+
+    get height() {
+        return this._height;
+    },
+
+    set height(height) {
+        if (height == this._height)
+            return;
+        this._height = height;
+        this.notify('height');
+        this._queueRedraw();
+    },
+
+    get width() {
+        return this._width;
+    },
+
+    set width(width) {
+        if (width == this._width)
+            return;
+
+        this._width = width;
+        this.notify('width');
+        this._queueRedraw();
+    },
+
+    vfunc_get_preferred_width_for_height: function(forHeight) {
+        if (this._width < 0)
+            return this.parent(forHeight);
+        else
+            return [this._width, this._width];
+    },
+
+    vfunc_get_preferred_height_for_width: function(forWidth) {
+        if (this._height < 0)
+            return this.parent(forWidth);
+        else
+            return [this._height, this._height];
+    }
+});
+
 const MainWindow = new Lang.Class({
     Name: 'MainWindow',
 
@@ -54,6 +125,15 @@ const MainWindow = new Lang.Class({
             Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
         );
 
+        this._accountsMonitor = AccountsMonitor.getDefault();
+        this._accountsMonitor.connect('accounts-changed', Lang.bind(this,
+            function(am) {
+                let accounts = am.dupAccounts();
+                this._revealer.reveal_child = accounts.some(function(a) {
+                    return a.enabled;
+                });
+            }));
+
         this._roomManager = ChatroomManager.getDefault();
         this._roomManager.connect('active-changed',
                                   Lang.bind(this, this._activeRoomChanged));
@@ -104,7 +184,7 @@ const MainWindow = new Lang.Class({
 
     _onSizeAllocate: function(widget, allocation) {
         if (!this._isFullscreen && !this._isMaximized)
-            this._currentSize = [allocation.width, allocation.height];
+            this._currentSize = this.window.get_size(this);
     },
 
     _onDelete: function(widget, event) {
@@ -140,7 +220,6 @@ const MainWindow = new Lang.Class({
         this._membersChangedId = 0;
 
         this._room = room;
-        this._revealer.reveal_child = room != null;
 
         this._updateTitlebar();
 
