@@ -6,9 +6,18 @@ const Gtk = imports.gi.Gtk;
 const Pango = imports.gi.Pango;
 const Tp = imports.gi.TelepathyGLib;
 
+const AccountsMonitor = imports.accountsMonitor;
 const ChatroomManager = imports.chatroomManager;
 const Lang = imports.lang;
 
+function _onPopoverVisibleChanged(popover) {
+    let context = popover.relative_to.get_style_context();
+    if (popover.visible)
+        context.add_class('has-open-popup');
+    else
+        context.remove_class('has-open-popup');
+}
+
 const RoomRow = new Lang.Class({
     Name: 'RoomRow',
 
@@ -17,6 +26,7 @@ const RoomRow = new Lang.Class({
 
         let app = Gio.Application.get_default();
         this.widget.room = room;
+        this.widget.account = room.account;
 
         this._popover = null;
 
@@ -106,14 +116,7 @@ const RoomRow = new Lang.Class({
                         'app.leave-room(("%s", ""))'.format(room.id));
 
             this._popover = Gtk.Popover.new_from_model(this.widget, menu);
-            this._popover.connect('notify::visible', Lang.bind(this,
-                function() {
-                    let roomContext = this.widget.get_style_context();
-                    if (this._popover.visible)
-                        roomContext.add_class('has-open-popup');
-                    else
-                        roomContext.remove_class('has-open-popup');
-                }));
+            this._popover.connect('notify::visible', _onPopoverVisibleChanged);
             this._popover.position = Gtk.PositionType.BOTTOM;
         }
         this._popover.show();
@@ -163,9 +166,11 @@ const RoomListHeader = new Lang.Class({
     Template: 'resource:///org/gnome/Polari/room-list-header.ui',
     InternalChildren: ['label',
                        'iconStack',
-                       'errorPopover',
-                       'popoverLabel',
-                       'popoverButton',
+                       'popoverStatus',
+                       'popoverTitle',
+                       'popoverReconnect',
+                       'popoverRemove',
+                       'popoverProperties',
                        'spinner'],
 
     _init: function(params) {
@@ -177,11 +182,12 @@ const RoomListHeader = new Lang.Class({
 
         this.parent(params);
 
-        this._errorPopover.relative_to = this._iconStack;
-        this._popoverButton.connect('clicked', Lang.bind(this,
-            function() {
-                this._errorPopover.hide();
-            }));
+        this.popover.connect('notify::visible', _onPopoverVisibleChanged);
+
+        let target = new GLib.Variant('o', this._account.get_object_path());
+        this._popoverReconnect.action_target = target;
+        this._popoverRemove.action_target = target;
+        this._popoverProperties.action_target = target;
 
         let displayNameChangedId =
             this._account.connect('notify::display-name',
@@ -190,8 +196,8 @@ const RoomListHeader = new Lang.Class({
 
         let connectionStatusChangedId =
             this._account.connect('notify::connection-status',
-                                  Lang.bind(this, this._updateConnectionStatusIcon));
-        this._updateConnectionStatusIcon();
+                                  Lang.bind(this, this._onConnectionStatusChanged));
+        this._onConnectionStatusChanged();
 
         this.connect('destroy', Lang.bind(this, function() {
             this._account.disconnect(displayNameChangedId);
@@ -214,80 +220,102 @@ const RoomListHeader = new Lang.Class({
         this.get_accessible().set_name(accessibleName);
     },
 
-    _updateConnectionStatusIcon: function() {
+    /* hack: Handle primary and secondary button interchangeably */
+    vfunc_button_press_event: function(event) {
+        if (event.button == Gdk.BUTTON_SECONDARY)
+            event.button = Gdk.BUTTON_PRIMARY;
+        return this.parent(event);
+    },
+
+    vfunc_button_release_event: function(event) {
+        if (event.button == Gdk.BUTTON_SECONDARY)
+            event.button = Gdk.BUTTON_PRIMARY;
+        return this.parent(event);
+    },
+
+    _onConnectionStatusChanged: function() {
         let status = this._account.connection_status;
         let reason = this._account.connection_status_reason;
         let isError = (status == Tp.ConnectionStatus.DISCONNECTED &&
                        reason != Tp.ConnectionStatusReason.REQUESTED);
-
         let child = 'none';
         if (status == Tp.ConnectionStatus.CONNECTING) {
             if (this._networkMonitor.network_available)
                 child = 'connecting';
         } else if (isError) {
             child = 'error';
-            switch (this._account.connection_error) {
-
-                case Tp.error_get_dbus_name(Tp.Error.CONNECTION_REFUSED):
-                case Tp.error_get_dbus_name(Tp.Error.NETWORK_ERROR): {
-                    this._popoverLabel.label = _("Please check your connection details.")
-
-                    this._popoverButton.label =  _("Edit Connection");
-                    this._popoverButton.action_name = 'app.edit-connection';
-                    this._popoverButton.action_target = new GLib.Variant('o', this._account.get_object_path());
-                    break;
-                }
-
-                case Tp.error_get_dbus_name(Tp.Error.CERT_REVOKED):
-                case Tp.error_get_dbus_name(Tp.Error.CERT_INSECURE):
-                case Tp.error_get_dbus_name(Tp.Error.CERT_LIMIT_EXCEEDED):
-                case Tp.error_get_dbus_name(Tp.Error.CERT_INVALID):
-                case Tp.error_get_dbus_name(Tp.Error.ENCRYPTION_ERROR):
-                case Tp.error_get_dbus_name(Tp.Error.CERT_NOT_PROVIDED):
-                case Tp.error_get_dbus_name(Tp.Error.ENCRYPTION_NOT_AVAILABLE):
-                case Tp.error_get_dbus_name(Tp.Error.CERT_UNTRUSTED):
-                case Tp.error_get_dbus_name(Tp.Error.CERT_EXPIRED):
-                case Tp.error_get_dbus_name(Tp.Error.CERT_NOT_ACTIVATED):
-                case Tp.error_get_dbus_name(Tp.Error.CERT_HOSTNAME_MISMATCH):
-                case Tp.error_get_dbus_name(Tp.Error.CERT_FINGERPRINT_MISMATCH):
-                case Tp.error_get_dbus_name(Tp.Error.CERT_SELF_SIGNED): {
-                    this._popoverLabel.label = _("Could not make connection in a safe way.");
-                    this._popoverButton.label =  _("Edit Connection");
-                    this._popoverButton.action_name = 'app.edit-connection';
-                    this._popoverButton.action_target = GLib.Variant.new('o', this._account.get_object_path());
-                    break;
-                }
-
-                case Tp.error_get_dbus_name(Tp.Error.AUTHENTICATION_FAILED): {
-                    this._popoverLabel.label = _("Authentication failed.");
-                    this._popoverButton.label = _("Try again");
-                    this._popoverButton.action_name = 'app.reconnect-account';
-                    this._popoverButton.action_target = GLib.Variant.new('o', this._account.get_object_path());
-                    break;
-                }
-
-                case Tp.error_get_dbus_name(Tp.Error.CONNECTION_FAILED):
-                case Tp.error_get_dbus_name(Tp.Error.CONNECTION_LOST):
-                case Tp.error_get_dbus_name(Tp.Error.CONNECTION_REPLACED):
-                case Tp.error_get_dbus_name(Tp.Error.SERVICE_BUSY): {
-                    this._popoverLabel.label = _("The server is busy.");
-                    this._popoverButton.label = _("Try again");
-                    this._popoverButton.action_name = 'app.reconnect-account';
-                    this._popoverButton.action_target = GLib.Variant.new('o', this._account.get_object_path());
-                    break;
-                }
-
-                default:
-                    this._popoverLabel.label = _("Failed to connect for an unknown reason.");
-                    this._popoverButton.label = _("Try again");
-                    this._popoverButton.action_name = 'app.reconnect-account';
-                    this._popoverButton.action_target = GLib.Variant.new('o', this._account.get_object_path());
-                    break;
-            }
         }
-        this.sensitive = isError;
         this._iconStack.visible_child_name = child;
         this._spinner.active = (child == 'connecting');
+
+        this._popoverTitle.use_markup = isError;
+        this._popoverStatus.use_markup = !isError;
+
+        if (!isError) {
+            let styleContext = this._popoverStatus.get_style_context();
+            styleContext.add_class('dim-label');
+
+            let params = this._account.dup_parameters_vardict().deep_unpack();
+            let server = params['server'].deep_unpack();
+            let accountName = this._account.display_name;
+
+            /* Translators: This is an account name followed by a
+               server address, e.g. "GNOME (irc.gnome.org)" */
+            let fullTitle = _("%s (%s)").format(accountName, server);
+            this._popoverTitle.label = (accountName == server) ? accountName : fullTitle;
+            this._popoverStatus.label = '<sup>' + this._getStatusLabel() + '</sup>';
+        } else {
+            let styleContext = this._popoverStatus.get_style_context();
+            styleContext.remove_class('dim-label');
+
+            this._popoverTitle.label = '<b>' + _("Connection Problem") + '</b>';
+            this._popoverStatus.label = this._getErrorLabel();
+        }
+    },
+
+    _getStatusLabel: function() {
+        switch (this._account.connection_status) {
+            case Tp.ConnectionStatus.CONNECTED:
+                return _("Connected");
+            case Tp.ConnectionStatus.CONNECTING:
+                return _("Connecting...");
+            case Tp.ConnectionStatus.DISCONNECTED:
+                return _("Offline");
+            default:
+                return _("Unknown");
+        }
+    },
+
+    _getErrorLabel: function() {
+        switch (this._account.connection_error) {
+
+            case Tp.error_get_dbus_name(Tp.Error.CERT_REVOKED):
+            case Tp.error_get_dbus_name(Tp.Error.CERT_INSECURE):
+            case Tp.error_get_dbus_name(Tp.Error.CERT_LIMIT_EXCEEDED):
+            case Tp.error_get_dbus_name(Tp.Error.CERT_INVALID):
+            case Tp.error_get_dbus_name(Tp.Error.ENCRYPTION_ERROR):
+            case Tp.error_get_dbus_name(Tp.Error.CERT_NOT_PROVIDED):
+            case Tp.error_get_dbus_name(Tp.Error.ENCRYPTION_NOT_AVAILABLE):
+            case Tp.error_get_dbus_name(Tp.Error.CERT_UNTRUSTED):
+            case Tp.error_get_dbus_name(Tp.Error.CERT_EXPIRED):
+            case Tp.error_get_dbus_name(Tp.Error.CERT_NOT_ACTIVATED):
+            case Tp.error_get_dbus_name(Tp.Error.CERT_HOSTNAME_MISMATCH):
+            case Tp.error_get_dbus_name(Tp.Error.CERT_FINGERPRINT_MISMATCH):
+            case Tp.error_get_dbus_name(Tp.Error.CERT_SELF_SIGNED):
+                return _("Could not connect to %s in a safe way.").format(this._account.display_name);
+
+            case Tp.error_get_dbus_name(Tp.Error.AUTHENTICATION_FAILED):
+                return _("Could not connect to %s. Authentication failed.").format(this._account.display_name);
+
+            case Tp.error_get_dbus_name(Tp.Error.CONNECTION_FAILED):
+            case Tp.error_get_dbus_name(Tp.Error.CONNECTION_LOST):
+            case Tp.error_get_dbus_name(Tp.Error.CONNECTION_REPLACED):
+            case Tp.error_get_dbus_name(Tp.Error.SERVICE_BUSY):
+                return _("Could not connect to %s. The server is busy.").format(this._account.display_name);
+
+            default:
+                return _("Could not connect to %s.").format(this._account.display_name);
+        }
     },
 });
 
@@ -302,6 +330,7 @@ const RoomList = new Lang.Class({
         this.widget.set_header_func(Lang.bind(this, this._updateHeader));
         this.widget.set_sort_func(Lang.bind(this, this._sort));
 
+        this._placeholders = {};
         this._roomRows = {};
         this._selectedRows = 0;
         this._selectionMode = false;
@@ -309,6 +338,27 @@ const RoomList = new Lang.Class({
         this.widget.connect('row-selected',
                             Lang.bind(this, this._onRowSelected));
 
+        this._accountsMonitor = AccountsMonitor.getDefault();
+        this._accountsMonitor.connect('account-manager-prepared', Lang.bind(this,
+            function(mon, am) {
+                let accounts = this._accountsMonitor.dupAccounts();
+                for (let i = 0; i < accounts.length; i++)
+                    this._accountAdded(mon, accounts[i]);
+
+                am.connect('account-enabled', Lang.bind(this,
+                    function(am, account) {
+                        this._updatePlaceholderVisibility(account);
+                    }));
+                am.connect('account-disabled', Lang.bind(this,
+                    function(am, account) {
+                        this._updatePlaceholderVisibility(account);
+                    }));
+            }));
+        this._accountsMonitor.connect('account-added',
+                                      Lang.bind(this, this._accountAdded));
+        this._accountsMonitor.connect('account-removed',
+                                      Lang.bind(this, this._accountRemoved));
+
         this._roomManager = ChatroomManager.getDefault();
         this._roomManager.connect('room-added',
                                   Lang.bind(this, this._roomAdded));
@@ -371,7 +421,12 @@ const RoomList = new Lang.Class({
         if (!current)
             return;
         let inc = direction == Gtk.DirectionType.UP ? -1 : 1;
-        let row = this.widget.get_row_at_index(current.get_index() + inc);
+        let index = current.get_index();
+        let row;
+        do {
+            index += inc;
+            row = this.widget.get_row_at_index(index);
+        } while (row && !row.room);
         if (row)
             this.widget.select_row(row);
     },
@@ -402,6 +457,35 @@ const RoomList = new Lang.Class({
             this.widget.select_row(selected);
     },
 
+    _accountAdded: function(am, account) {
+        if (this._placeholders[account])
+            return;
+
+        let placeholder = new Gtk.ListBoxRow({ selectable: false,
+                                               activatable: false });
+        placeholder.account = account;
+
+        this._placeholders[account] = placeholder;
+        this.widget.add(placeholder);
+
+        placeholder.connect('notify::visible', Lang.bind(this,
+            function() {
+                this.widget.invalidate_sort();
+            }));
+
+        this._updatePlaceholderVisibility(account);
+    },
+
+    _accountRemoved: function(am, account) {
+        let placeholder = this._placeholders[account];
+
+        if (!placeholder)
+            return;
+
+        delete this._placeholders[account];
+        placeholder.destroy();
+    },
+
     _roomAdded: function(roomManager, room) {
         let roomRow = new RoomRow(room);
         this.widget.add(roomRow.widget);
@@ -411,6 +495,7 @@ const RoomList = new Lang.Class({
             function(w) {
                 delete this._roomRows[w.room.id];
             }));
+        this._placeholders[room.account].hide();
     },
 
     _roomRemoved: function(roomManager, room) {
@@ -421,6 +506,21 @@ const RoomList = new Lang.Class({
         this._moveSelectionFromRow(roomRow.widget);
         roomRow.widget.destroy();
         delete this._roomRows[room.id];
+        this._updatePlaceholderVisibility(room.account);
+    },
+
+    _updatePlaceholderVisibility: function(account) {
+        if (!account.enabled) {
+            this._placeholders[account].hide();
+            return;
+        }
+
+        let ids = Object.keys(this._roomRows);
+        let hasRooms = ids.some(Lang.bind(this,
+            function(id) {
+                return this._roomRows[id].widget.account == account;
+            }));
+        this._placeholders[account].visible = !hasRooms;
     },
 
     _activeRoomChanged: function(roomManager, room) {
@@ -444,7 +544,7 @@ const RoomList = new Lang.Class({
 
     _updateHeader: function(row, before) {
         let getAccount = function(row) {
-            return row ? row.room.account : null;
+            return row ? row.account : null;
         };
         let beforeAccount = getAccount(before);
         let account = getAccount(row);
@@ -462,6 +562,30 @@ const RoomList = new Lang.Class({
     },
 
     _sort: function(row1, row2) {
-        return row1.room.compare(row2.room);
+        let account1 = row1.account;
+        let account2 = row2.account;
+
+        let hasRooms1 = !this._placeholders[account1].visible;
+        let hasRooms2 = !this._placeholders[account2].visible;
+
+        if (hasRooms1 != hasRooms2)
+            return hasRooms1 ? -1 : 1;
+
+        if (account1 != account2)
+            return account1.display_name.localeCompare(account2.display_name);
+
+        let room1 = row1.room;
+        let room2 = row2.room;
+
+        if (!room1)
+            return -1;
+
+        if (!room2)
+            return 1;
+
+        if (room1.type != room2.type)
+            return room1.type == Tp.HandleType.ROOM ? -1 : 1;
+
+        return room1.display_name.localeCompare(room2.display_name);
     }
 });
