@@ -6,6 +6,7 @@ const ChatroomManager = imports.chatroomManager;
 const ChatView = imports.chatView;
 const EntryArea = imports.entryArea;
 const Lang = imports.lang;
+const Signals = imports.signals;
 
 const RoomStack = new Lang.Class({
     Name: 'RoomStack',
@@ -66,7 +67,6 @@ const ChatPlaceholder = new Lang.Class({
 
     _init: function() {
         this._accountsMonitor = AccountsMonitor.getDefault();
-        this._accountsMonitor.connect('accounts-changed', Lang.bind(this, this._checkAccounts));
 
         let image = new Gtk.Image({ icon_name: 'polari-symbolic',
                                       pixel_size: 96, halign: Gtk.Align.END,
@@ -77,54 +77,22 @@ const ChatPlaceholder = new Lang.Class({
         title.label = '<span letter_spacing="4500">%s</span>'.format(_("Polari"));
         title.get_style_context().add_class('polari-background-title');
 
-        this._description = new Gtk.Label({ halign: Gtk.Align.CENTER, wrap: true,
-                                            margin_top: 24, use_markup: true });
-        this._description.get_style_context().add_class('polari-background-description');
-
-        this._instruction = new Gtk.Label({ halign: Gtk.Align.CENTER,
-                                            use_markup: true, wrap: true });
-        this._instruction.connect('activate-link', Lang.bind(this,
-            function(label, actionName) {
-                let app = Gio.Application.get_default();
-                let action = app.lookup_action(actionName);
-                if (action)
-                    action.activate(null);
-                return action != null;
-            }));
+        let description = new Gtk.Label({ label: _("Join a room using the + button."),
+                                          halign: Gtk.Align.CENTER, wrap: true,
+                                          margin_top: 24, use_markup: true });
+        description.get_style_context().add_class('polari-background-description');
 
         this.widget = new Gtk.Grid({ column_homogeneous: true, can_focus: false,
-                                     column_spacing: 18, vexpand: true,
+                                     column_spacing: 18, hexpand: true, vexpand: true,
                                      valign: Gtk.Align.CENTER });
         this.widget.get_style_context().add_class('polari-background');
         this.widget.attach(image, 0, 0, 1, 1);
         this.widget.attach(title, 1, 0, 1, 1);
-        this.widget.attach(this._description, 0, 1, 2, 1);
-        this.widget.attach(this._instruction, 0, 2, 2, 1);
+        this.widget.attach(description, 0, 1, 2, 1);
         this.widget.show_all();
-
-        this._checkAccounts();
-    },
-
-    _checkAccounts: function() {
-        let accounts = this._accountsMonitor.dupAccounts();
-        if (accounts.length == 0) {
-            this._description.label = _("Begin chatting by adding a new connection.");
-            /* translators: This will be used in the phrase: "Open Connections in the application menu" */
-            let href = '<a href="connections">%s</a>'.format(_("Connections"));
-            this._instruction.label = _("Open %s in the application menu.").format(href);
-
-        } else if (accounts.some(function(a) { return a.enabled; })) {
-            this._description.label = _("Join a room using the + button.");
-            this._instruction.label = '';
-
-        } else {
-            this._description.label = _("Your connections are disabled.");
-            /* translators: This will be used in the phrase: "Enable them by opening Connections in the application menu" */
-            let href = '<a href="connections">%s</a>'.format(_("Connections"));
-            this._instruction.label = _("Enable them by opening %s in the application menu.").format(href);
-        }
-    },
+    }
 });
+Signals.addSignalMethods(ChatPlaceholder.prototype);
 
 const RoomView = new Lang.Class({
     Name: 'RoomView',
@@ -132,22 +100,23 @@ const RoomView = new Lang.Class({
     _init: function(room) {
         this._view = room ? new ChatView.ChatView(room)
                           : new ChatPlaceholder();
-
-        this._entryArea = new EntryArea.EntryArea(room);
+        this._view.connect('max-nick-chars-changed', Lang.bind(this,
+            function() {
+                this.inputWidget.maxNickChars = this._view.maxNickChars;
+            }));
 
         this.widget = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL });
         this.widget.add(this._view.widget);
 
-        this.inputWidget = new Gtk.Frame({ shadow_type: Gtk.ShadowType.NONE });
-        this.inputWidget.get_style_context().add_class('polari-input-area');
+        this.inputWidget = room ? new EntryArea.EntryArea({ room: room,
+                                                            sensitive: false })
+                                : new Gtk.Box();
         this.widget.add(this.inputWidget);
 
-        this.inputWidget.add(this._entryArea.widget);
-
         this.widget.show_all();
     },
 
     set inputSensitive(sensitive) {
-        this._entryArea.widget.sensitive = sensitive;
+        this.inputWidget.sensitive = sensitive;
     }
 });
